
!
! A stripped down bootsect to be overlaid on bytes 0-496 of
! the original bootsect.
! All this one does is make a copy of the default Disk Parameter Block
! in the BIOS and modify dpb[4] to 36 so that multi-sector reads
! will succeed.
!
! Modified from
!	bootsect.s		Copyright (C) 1991, 1992 Linus Torvalds
!
#include <linuxmt/config.h>

/*
 *	Many BIOS's default disk parameter tables will not 
 *	recognize multi-sector reads beyond the maximum sector number
 *	specified in the default diskette parameter tables - this may
 *	mean 7 sectors in some cases.
 *
 *	Since single sector reads are slow and out of the question,
 *	we must take care of this by creating new parameter tables
 *	(for the first disk) in RAM.  We will set the maximum sector
 *	count to 36 - the most we will encounter on an ED 2.88.  
 *
 *	High doesn't hurt.  Low does.
 *
 *	Segments are as follows: ds=es=ss - INITSEG,
 */

	.text

INITSEG   = DEF_INITSEG			! we move boot here - out of the way
SETUPSEG  = DEF_SETUPSEG

	xor	ax,ax
	mov	es,ax			! access segment 0
	mov	bx,#0x78		
! 0:bx is parameter table address
	seg es
	lds	si,[bx]			! loads ds and si

	mov	ax,#INITSEG
	mov	es,ax
	mov	di,#0x4000-12	
! 0x4000 is arbitrary value >= length of
! bootsect + length of setup + room for stack
! 12 is disk parm size

	mov	cl,#6			
! copy 12 bytes
	cld
	push	di			! keep a copy for later
	rep
! ds:si is source, es:di is dest
	movsw
	pop	di

	seg es
	movb	4[di],#36		! patch sector count

	xor	ax,ax
	mov	ds,ax			! access segment 0
	mov	[bx],di
	mov	2[bx],es

	jmpi	0,SETUPSEG
