! first.S  -  primary boot loader
!
! Version 1.0 <Sun Jan 19 17:00:00 1997>
!
! Copyright (C) 1996    Gero Kuhlmann   <gero@gkminix.han.de>
!                   and Markus Gutschke <gutschk@math.uni-muenster.de>
!
!  This program is free software; you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation; either version 2 of the License, or
!  any later version.
!
!  This program is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with this program; if not, write to the Free Software
!  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#ifndef ASM_DEBUG
#undef ASM_DEBUG
#endif
#include "first.inc"

! Note on the following code: this has been written to run in real-86-mode.

	.text

!	.globl	_main

	.org	0

_main:	mov	dx,ds
	mov	ax,cs			! set DS and ES
	mov	ds,ax
	mov	oldES,es
	mov	oldDS,dx		! save old register values in case
	mov	oldBP,bp		! we have to return to the boot rom
	mov	oldSI,si
	mov	oldDI,di
	mov	bp,sp
	mov	ax,[bp+4]
	mov	[header+0],ax		! load the address of the boot header
	mov	ax,[bp+6]
	mov	[header+2],ax
	mov	ax,[bp+8]
	mov	[bootp+0],ax		! load the address of the bootp block
	mov	ax,[bp+10]
	mov	[bootp+2],ax

! Tell the user who we are and that we started running

	mov	si,#sigmsg
	call	prnstr

! Check if the boot image header is correct.

	les	bx,header
	mov	si,#bmerr		! prepare for correct error message
	seg	es
	mov	ax,BOOT_HD_MAGIC+0[bx]
	cmp	ax,[bmagic+0]		! compare boot rom magic numbers
	jne	doerr1
	seg	es
	mov	ax,BOOT_HD_MAGIC+2[bx]
	cmp	ax,[bmagic+2]
	jne	doerr1

	mov	si,#vmerr		! prepare for correct error message
	seg	es
	mov	al,BOOT_HD_LENGTH[bx]
	mov	cl,#4
	shr	al,cl
	and	al,#0x0F
	cmp	al,#VENDOR_SIZE		! check vendor ID size
	jne	doerr1
	xor	di,di
dovmag:	mov	al,vmagic[di]		! check vendor ID
	or	al,al
	jz	chkmem			! vendor ID ok, continue
	seg	es
	cmp	al,BOOT_HD_VENDOR[bx+di]
	jne	doerr1
	inc	di
	jmp	dovmag

doerr0:	mov	ax,cs
	mov	ds,ax
	mov	si,#bootmsg
doerr1:	call	prnstr			! in case of error return to the
	mov	si,oldSI		! boot rom with registers set
	mov	di,oldDI		! correctly
	mov	bp,oldBP
	mov	es,oldES
	mov	ds,oldDS
	retf

! Get the size of the base memory from the BIOS. We need at least
! 612 kB, which is the amount of base RAM below the boot rom memory
! area starting at 0x98000, plus the amount of static RAM needed for
! stack. Note that this code just checks for the absolut minimum which
! is required. The actual amount of memory needed is determined
! lateron after determining the real base of the static memory area.

chkmem:	int	0x12			! get memory size in kB
	cmp	ax,#MIN_MEM
	mov	si,#memerr		! check if enough
	jb	doerr1
	mov	avlmem,ax		! save it for later

! Looks good. Now find the lowest usable spot of memory and check
! if there is still enough to hold all necessary dynamic memory.

	mov	bx,#BOOT_LD_ILENGTH	! find the highest used memory
	call	fndmem			! block
#if defined(ASM_DEBUG) || defined(ASM_DEBUG_VERBOSE)
	push	ax
	mov	dx,ax
	mov	si,#dynmsg
	call	prnstr
	mov	ax,dx
	call	prnwrd			! lets see what we got here
	mov	si,#crlf
	call	prnstr
	pop	ax
#endif
	mov	dx,ax
	mov	ax,cs
	mov	si,#recerr
	cmp	dx,ax			! oops, shouldnt be below us
	jbe	doerr3
	mov	bx,#MAX_MEM-(MAX_DYNAMIC_LEN)/16-1
	mov	si,#memerr
	cmp	dx,bx			! check if enough memory
	jbe	setptr
doerr3:	jmp	doerr1

! We can now setup the pointers to all dynamic data structures. Note
! that the bootp memory block comes last so that we can use larger
! bootp records if required (and supported by the server), up until
! the beginning of the boot rom area (0x98000).

setptr:	mov	[botptr+2],dx
	mov	word ptr [botptr],#0	! then update the offsets

! Before we can use the memory area of the boot rom we have to secure
! all interesting data, i.e. the bootp record. First check to see if we got
! a bootp reply here, and copy it to the dynamic memory area.

	cld
	xor	dx,dx
	les	di,bootp
	seg	es
	mov	al,BOOTP_OP[di]		! op code must indicate reply
	cmp	al,#BOOTP_REPLY
	jne	dobot2			! it isnt
	add	di,#BOOTP_VEND
	mov	bx,di
	mov	si,#pmagic		! compare vendor ID
dobot1:	mov	di,bx
	mov	cx,#BOOTP_MAGIC_LEN
	repe
	cmpsb
	jz	dobot3			! vendor ID is valid
	add	si,cx
	cmp	byte ptr [si],#0	! check next vendor ID
	jne	dobot1
dobot2:	jmp	nobot2			! vendor ID not found

dobot3:	sub	si,#BOOTP_MAGIC_LEN
	sub	si,#pmagic
	mov	botid,si		! save vendor ID offset
	mov	ax,si
	push	ds
	mov	bp,sp
	les	di,botptr
	lds	si,bootp
	mov	bx,si
	mov	dx,#BOOTP_SIZE
	or	ax,ax			! if not RFC vendor ID the bootp
	jnz	dobot7			! record has fixed length

	xor	cx,cx
	add	si,#BOOTP_VEND + BOOTP_MAGIC_LEN
dobot4:	lodsb
	cmp	al,#BOOTP_RFC_NOP	! handle NOP tag
	jnz	dobot5
	inc	cx
	cmp	cx,#16			! more than 16 NOP tags is VERY unusual
	jae	dobot7			! so the bootp record maybe broken
	jmp	dobot4			! loop to next tag

doerr6:	jmp	doerr3
	
dobot5:	cmp	al,#BOOTP_RFC_END	! handle END tag
	jnz	dobot6
	mov	dx,si
	sub	dx,bx			! compute length of bootp record
	cmp	dx,#BOOTP_SIZE
	jae	dobot7
	mov	dx,#BOOTP_SIZE		! use minimum size
	jmp	dobot7

nobot2:	jmp	nobotp
	
dobot6:	lodsb				! handle all other tags
	mov	cl,al
	xor	ch,ch
	add	si,cx			! jump to next tag
	xor	cx,cx			! reset NOP counter
	jmp	dobot4			! proceed with next tag

dobot7:	mov	si,#btperr		! determine amount of dynamic memory
	mov	ax,#MAX_MEM		! available for bootp record
	mov	cx,es			! [botptr+2]
	sub	ax,cx
	test	ax,#0xF000
	jnz	dobot8
	mov	cl,#4
	shl	ax,cl
	mov	cx,di			! [botptr+0]
	sub	ax,cx
	cmp	dx,ax			! check if bootp record fits into
	jb	dobot8			! dynamic memory area
doerr7:	jmp	doerr6

dobot8:	mov	cx,dx
	mov	si,bx			! restore source pointer
	rep
	movsb				! save the bootp record

	pop	ds

#if defined(ASM_DEBUG) || defined(ASM_DEBUG_VERBOSE)
	mov	si,#btpmsg
	call	prnstr
	mov	ax,botid
	call	prnwrd			! let the user know
	mov	si,#crlf
	call	prnstr
#endif

nobotp:	mov	botlen,dx		! set length of bootp record

! Go ahead now and try to load the boot sector. First we have to extract the
! device name from the filename of the loaded image.

devicename:	
	mov	al,#BOOTP_RFC_VNDID	! check for 'Eth' vendor ID
	call	gettag
	cmp	cx,#6			! there have to be at least 6 bytes
	jb	dnerr			! in this tag
	seg	es
	cmp	word ptr 0[di],#VNDID_1
	jnz	dnerr
	seg	es
	cmp	word ptr 2[di],#VNDID_2
	jnz	dnerr
	mov	al,#BOOTP_RFC_VNDSEL	! get user selection
	call	gettag
	cmp	cx,#1
	jae	dnimg
dnerr:	mov	si,#nameerr
doerr8:	jmp	doerr7
dnimg:	seg	es
	mov	al,byte ptr 0[di]	! get selected image descripton
	call	gettag
	jcxz	dnerr
	seg	es
	mov	cl,byte ptr -1[di]
	cld				! scan for filename field
	mov	bx,#3
	mov	ax,#0x3A		! count colons
dnloop1:repne
	scasb
	or	cx,cx
	jz	dnerr
	dec	bx
	jnz	dnloop1
	push	ds
	mov	ax,es
	mov	ds,ax
	mov	si,di
dnloop2:lodsb
	or	al,al
	jz	dndone
	cmp	al,#0x3A		! ':'
	jz	dndone
	cmp	al,#0x2F		! '/'
	jnz	dncont1
	mov	di,si
dncont1:loop	dnloop2
dndone:	pop	ds			! es:di points to the basename, now

#ifdef ASM_DEBUG_VERBOSE
	push	di
	push	ds
	mov	si,#bnmsg
	call	prnstr
	mov	si,di
	mov	ax,es
	mov	ds,ax
dndbg1:	lodsb
	or	al,al
	jz	dndbg2
	cmp	al,#0x3A
	jz	dndbg2
	call	prnchr
	jmp	dndbg1
dndbg2:	pop	ds
	mov	si,#crlf
	call	prnstr
	pop	di
#endif
	call	decodestr		! determine device number, partion id
	jc	dnerr

	xor	ah,ah			! reset disk system

! First load the master boot record

	mov	ax,#BOOTSEG		! destination segment in ES
	mov	es,ax
	mov	cx,#1			! load first sector on track 0
	xor	dh,dh			! head 0
	call	reads1
	jnc	ldpart
lderr:	mov	si,#ldmsg
	jmp	doerr8

! Next load the first sector of the partition. If we are to load from a
! partition in an extended partition, we have to read the extended partition
! table first.

ldpart:	mov	bl,partnum
	or	bl,bl			! check if that is all
	jz	ldphys
	cmp	bl,#5			! check if we have to load from an
	jb	ldprim			! extended partition
	mov	bx,#PART_OFS		! search for an extended partition
	mov	cx,#ENTRY_NUM
ldloop:	seg	es
	cmp	byte ptr PART_TYPE_OFS[bx],#EXT_PART_TYPE
	je	ldext
	add	bx,#ENTRY_LEN		! continue with next entry
	loop	ldloop
	jmp	lderr
ldext:	call	readsect		! read extended partition table
	jc	lderr
	mov	bl,partnum		! compute partition number in extended
	sub	bl,#4			! table
ldprim:	dec	bl			! partition numbers start with 1
	xor	bh,bh
	mov	cl,#4
	shl	bx,cl			! compute offset into partition table
	add	bx,#PART_OFS
	call	readsect		! read first sector of boot partition
	jc	lderr
ldphys:

! At this point all dynamic data, which is only necessary for this boot
! loader, has been saved. Now we can go ahead and acquire memory for every-
! thing which must remain intact when we call the loaded bootsector, i.e.
! the stack. We can now use the boot rom data area for this, so first find
! the highest used reserved memory segment from all load records, and setup
! the stack.

	mov	bx,#BOOT_LD_MLENGTH	! find the highest reserved memory
	call	fndmem			! block
	cmp	ax,#MAX_MEM		! should be at least in the boot rom
	jae	setm1			! data area
	mov	ax,#MAX_MEM
setm1:	mov	dx,[botptr+0]		! the static data should not interfere
	add	dx,botlen		! with the dynamic data area. botptr
	add	dx,#0x000F		! is the last data field in the dynamic
	mov	cl,#4			! area, so compute its ending segment
	shr	dx,cl			! and use that as the static base.
	add	dx,[botptr+2]
	inc	dx
	cmp	ax,dx			! set end of dynamic data area as base
	jae	setm3			! of static area in case of conflict
	mov	ax,dx
setm3:
	
#if defined(ASM_DEBUG) || defined(ASM_DEBUG_VERBOSE)
	push	ax
	mov	dx,ax
	mov	si,#stamsg
	call	prnstr
	mov	ax,dx
	call	prnwrd			! lets see what we got here
	mov	si,#crlf
	call	prnstr
	pop	ax
#endif

! AX contains the first segment usable for static memory.

#if defined(ASM_DEBUG) || defined(ASM_DEBUG_VERBOSE)
	push	ax
	mov	dx,ax
	mov	si,#stkmsg
	call	prnstr
	mov	ax,dx
	call	prnwrd
	mov	al,#0x3A
	call	prnchr			! show the new stack pointer
	mov	ax,#STACK_SIZE		! on the console
	call	prnwrd
	mov	si,#crlf
	call	prnstr
	pop	ax
#endif

! Now set the stack. From this point on there is no return to the boot rom.

	cli				! disable interrupts during stack setup
	mov	sp,#STACK_SIZE		! set the stack
	mov	ss,ax
	sti				! it is safe again

! At this point the memory layout should look like this, assuming default
! values for all load records set by mknbi:
!
! 0x92200-0x931FF	  4.0 kB	primary boot loader - this program
! 0x93200-0x933FF	  0.5 kB	boot image header
! 0x93400-0x935FF	  0.5 kB	dynamic memory - bootp record
! 0x93600-0x97FFF	 18.5 kB	(free)
! 0x98000-0x98FFF	  4.0 kB	static memory - stack
! 0x99000-0x9FFFF	 28.0 kB	(free)
!
	
#if (defined(ASM_DEBUG) || defined(ASM_DEBUG_VERBOSE)) && \
    (defined(ASM_FREEZE_AFTER_INIT) || defined(ASM_KEYPRESS_AFTER_INIT))
	mov	si,#waitmsg
	call	prnstr
#endif
#ifdef ASM_FREEZE_AFTER_INIT
lop1:	jmp	lop1			! This is a feature, not a bug...
#endif
#ifdef ASM_KEYPRESS_AFTER_INIT
	xor	ah,ah
	int	0x16
#endif

	push	cs
	mov	ax,#doerr9
	push	ax
	mov	ax,#BOOTSEG
	push	ax
	xor	dx,dx
	push	dx
	retf

doerr9:	mov	ax,cs			! cannot return, because stack
	mov	ds,ax			! points into boot prom code
	mov	si,#bootmsg
	call	prnstr
errloop:jmp	errloop


!====================================================================
!
! Scan all load records, and find the highest used memory address,
! round it up to the nearest segment boundary and return it in AX.
! The offset to the length value in the load records is given in BX.
! Ignore all memory segments that have been loaded into extended memory!
!
! Changed registers: AX, DI, ES

fndmem:	push	cx
	push	dx
	les	di,header
	mov	ax,di
	add	ax,#0x000F
	and	ax,#0xFFF0		! round up to nearest segment
	mov	cl,#4
	shr	ax,cl
	mov	cx,es			! convert address of header into
	add	ax,cx			! segment representation, and add
	jc	fndm7			! the size of the header in order to
	add	ax,#BOOT_SIZE / 16	! get a starting value for the highest
	jnc	fndm8			! memory segment.
fndm7:	xor	ax,ax			! the highest segment is on the stack
fndm8:	push	ax

	seg	es
	mov	al,BOOT_HD_LENGTH[di]	! get length of image header
	call	getlen
	add	di,ax			! get the pointer to first load record
fndm1:	seg	es
	mov	al,BOOT_LD_FLAGS[di]		! get load record flags
	test	al,#BOOT_FLAG_B0 + BOOT_FLAG_B1	! we are presently unable
	jnz	fndm4				! to handle relative addrs
	seg	es
	mov	dx,BOOT_LD_ADDR+0[di]
	seg	es
	mov	ax,BOOT_LD_ADDR+2[di]	! get the load address
	seg	es
	add	dx,0[di+bx]		! add the length (offset in BX)
	seg	es
	adc	ax,2[di+bx]
	add	dx,#0x000F		! round up to the nearest segment
	adc	ax,#0
	and	dx,#0xFFF0
	test	ax,#0xFFF0		! only allow addresses in lower 1 MB
	jnz	fndm4

	mov	cl,#4			! convert address into segment address
	shr	dx,cl			! by shifting everything right by 4 bits
	mov	cl,#12
	shl	ax,cl
	or	ax,dx			! combine higher and lower word of addr
	pop	dx
	cmp	ax,dx			! is new address higher than old one?
	jbe	fndm3
	mov	dx,ax			! yes, save it
fndm3:	push	dx

fndm4:	seg	es
	mov	al,BOOT_LD_FLAGS[di]	! ok, thats it, now check if thats the
	test	al,#BOOT_FLAG_EOF	! last record
	jnz	fndm5
	seg	es
	mov 	al,BOOT_LD_LENGTH[di]	! no, get the address of the next one
	call	getlen
	add	di,ax
	jmp	fndm1
fndm5:	pop	ax			! return the highest used segment
	pop	dx
	pop	cx
	ret



!====================================================================
!
! Compute the length of a load record address from a length byte
! in AL. Return the offset in AX.
!
! Changed registers: AX

getlen:	push	cx
	mov 	ah,al
	mov 	cl,#4
	shr	ah,cl
	and	ax,#0x0f0f		! compute the total length in
	add	al,ah			! bytes from the length of the
	xor	ah,ah			! record and that of the vendor
	shl	ax,1			! information.
	shl	ax,1
	pop	cx
	ret



!====================================================================
!
! Find a vendor tag in the bootp record. The tag number is in AL.
! It returns the pointer to the tag string in ES:DI and the length
! of the string in CX. CX is zero if tag not found.
!
! Changed registers: AX, CX, DI, ES

gettag:	cld
	push	si
	push	dx
	xor	dx,dx
	cmp	botlen,#BOOTP_SIZE
	jb	gett8			! no bootp record available
	cmp	botid,#0		! use only RFC vendor information
	jne	gett8
	les	di,botptr
	mov	si,di
	add	si,botlen		! let ES:SI point to last bootp byte
	add	di,#BOOTP_VEND
	add	di,#BOOTP_MAGIC_LEN

	mov	ah,al			! save tag number
	xor	ch,ch
gett1:	seg	es
	mov	al,[di]			! load current vendor tag
	inc	di
	cmp	al,#BOOTP_RFC_NOP	! NOP tag --> continue
	je	gett1
	cmp	al,#BOOTP_RFC_END	! END tag --> abort
	je	gett8
	seg	es
	mov	cl,[di]			! get length of current tag
	inc	di
	cmp	al,ah			! we got our tag
	je	gett9
	add	di,cx			! proceed with next tag
	cmp	di,si
	jae	gett8			! got at end of bootp record
	jmp	gett1

gett8:	mov	cx,dx
gett9:	pop	dx
	pop	si
	ret



!====================================================================
!
! Print a string in DS:SI onto the console
!
! Changed registers: AL

prnstr:	push	si
	cld
prns1:	lodsb				! loop over all characters of
	or	al,al			! string
	jz	prns2
	call	prnchr			! print character
	jmp	prns1
prns2:	pop	si
	ret



!====================================================================
!
! Print hexadecimal values (in AX or AL) or characters onto the console
!
! Changed registers: AX

prnwrd:	push	ax
	mov	al,ah
	call	prnbyt			! print the upper byte
	pop	ax
prnbyt: push	ax
	shr	al,1			! prepare upper nibble
	shr	al,1
	shr	al,1
	shr	al,1
	call	prnnib			! print it
	pop	ax
prnnib:	and	al,#0x0F		! prepare lower nibble
	add	al,#0x30
	cmp	al,#0x39		! convert it into hex
	jle	prnchr
	add	al,#7
prnchr:	push	bx
	mov	ah,#0x0E		! print it
	mov	bl,#0x07
	xor	bh,bh
	int	0x10
	pop	bx
	ret

!====================================================================
!
! Decode device string in ES:DI
!
! Changed registers: AX, DI

decodestr:
! First decode the first two letters. The string has to start either
! with 'fd' or 'hd'. We do not do any other devices yet.
	seg	es
	cmp	byte ptr 1[di],#0x64	! second letter has to be a 'd'
	jne	decod0
	seg	es
	mov	ah,byte ptr 2[di]
	mov	al,#HD_BASE
	seg	es
	cmp	byte ptr 0[di],#0x68	! check for 'h' hard disk
	je	decod1
	mov	al,#FD_BASE
	seg	es
	cmp	byte ptr 0[di],#0x66	! check for 'f' floppy disk
	jne	decod0

! OK, we have to look for a floppy device. After the initial 'fd' a number
! has to follow. The maximum is 4 disk drives, so check that it is in the
! range 0 to 3.

	sub	ah,#0x30		! check for correct number
	jb	decod0
	cmp	ah,#3
	ja	decod0
	add	al,ah			! compute final device number
	xor	ah,ah			! partition number is always zero
	add	di,#3			! let DI point after last character
	jmp	decod2

! Now decode the harddisk description. The initial 'hd' has to be followed
! by a letter from 'a' to 'd' indicating the drive number, followed by a
! number indicating the partition number. If the number is missing, 0 is
! assumed, which means the whole disk, e.g. the master boot block.

decod0:	jmp	decod4
decod1:	sub	ah,#0x61		! check for correct letter
	jb	decod4
	cmp	ah,#3
	ja	decod4
	add	al,ah			! compute final device number
	add	di,#3			! let BX point after last character
	seg	es
	mov	ah,0[di]
	or	ah,ah			! end of string means "zero"
	jz	decod2
	sub	ah,#0x3a
	jz	decod2
	sub	ah,#0xf6		! check for correct partition
	jb	decod4
	cmp	ah,#8			! maximum number of 8 partitions
	ja	decod4
	inc	di

! Return with the global variables set correctly

decod2:	seg	es
	cmp	byte ptr 0[di],#0	! string has to be followed by a null
	jz	decod3
	seg	es
	cmp	byte ptr 0[di],#0x3a	! or colon
	jne	decod4
decod3:	mov	diskid,al		! set the global variables
	mov	partnum,ah
	clc				! return without error
	jmp	decod5

decod4:	stc				! return with error
decod5:	ret


!====================================================================
!
! Read first sector of partitions. reads1 is an additional subroutine entry
! point! BX points to partition table entry. ES is segment of bootblock buffer
!
! Changed registers: AX, BX, CX, DX

readsect:
	seg	es
	mov	al,PART_TYPE_OFS[bx]
	or	al,al			! check if partition table entry
	jz	reads3			! is valid
	seg	es
	mov	cx,PART_SECT_OFS[bx]	! get sector, track and head
	seg	es
	mov	dh,PART_HEAD_OFS[bx]	! number from partition table
reads1:	mov	dl,diskid
	mov	bx,#3			! retry count
reads2:	push	bx
	xor	bx,bx
	mov	ax,#0x201		! actually read the sector
#ifdef ASM_DEBUG_VERBOSE
	call	logread
#endif
	int	0x13
	jc	reads5
	seg	es
	cmp	word ptr BOOT_SIG_OFS[bx],#BOOT_SIG
	pop	bx
	je	reads4
reads3:	stc
reads4:	ret
reads5:	pop	bx
	dec	bx
	jnz	reads2
	ret

#ifdef ASM_DEBUG_VERBOSE
logread:push	ax
	push	si
	push	ax
	mov	si,#readmsg
	call	prnstr
	pop	ax
	call	prnwrd
	mov	al,#0x2c
	call	prnchr
	mov	ax,es
	call	prnwrd
	mov	al,#0x3a
	call	prnchr
	mov	ax,bx
	call	prnwrd
	mov	al,#0x2c
	call	prnchr
	mov	ax,cx
	call	prnwrd
	mov	al,#0x2c
	call	prnchr
	mov	ax,dx
	call	prnwrd
	mov	si,#crlf
	call	prnstr
	pop	si
	pop	ax
	ret
#endif	

!====================================================================
!
! String and constants definitions


! Startup signature

sigmsg:	.byte	0x0D, 0x0A
	.ascii	"Blockdevice Loader"
	.byte	0x0D, 0x0A
crlf:	.byte	0x0D, 0x0A
	.byte	0


! Magic numbers for boot record and bootp entry

bmagic:	.long	BOOT_MAGIC		! boot image magic number
vmagic:	.ascii	VENDOR_MAGIC		! vendor magic ID
	.byte	0			! end of vendor magic ID
pmagic:	.byte	BOOTP_MAGIC_RFC		! bootp magic ID for RFC 1048
	.byte	BOOTP_MAGIC_CMU		! bootp magic ID for CMU
	.byte	BOOTP_MAGIC_STA		! bootp magic ID for Stanford
	.byte	0


! Error messages

memerr:	.ascii	"Not enough memory"
	.byte	0x0D, 0x0A
	.byte	0

recerr:	.ascii	"Error in load record data"
	.byte	0x0D, 0x0A
	.byte	0

btperr:	.ascii	"BOOTP record too large"
	.byte	0x0D, 0x0A
	.byte	0

bmerr:	.ascii	"Invalid boot header magic number"
	.byte	0x0D, 0x0A
	.byte	0

vmerr:	.ascii	"Invalid vendor magic ID"
	.byte	0x0D, 0x0A
	.byte	0

nameerr:.ascii	"Could not find a valid blockdevice name in BOOTP record"
	.byte	0x0D, 0x0A
	.byte	0

ldmsg:	.ascii	"Error reading bootsector"
	.byte	0x0D, 0x0A
	.byte	0

bootmsg:.ascii	"Error executing bootsector"
	.byte	0x0D, 0x0A
	.byte	0

#if defined(ASM_DEBUG) || defined(ASM_DEBUG_VERBOSE)
! Some debug messages

btpmsg:	.ascii	"Using bootp record, vendor ID is "
	.byte	0

dynmsg:	.ascii	"Dynamic memory start: "
	.byte	0

stamsg:	.ascii	"Static memory start: "
	.byte	0

bnmsg:	.ascii	"Basename: "
	.byte	0
	
stkmsg:	.ascii	"Stack pointer set to: "
	.byte	0

waitmsg:.ascii	"Done!"
	.byte	0x0D, 0x0A
	.byte	0

readmsg:.ascii	"INT 13: "
	.byte	0
#endif



!====================================================================
!
! Variable definitions

header:	.long	0			! pointer to boot header from boot rom
bootp:	.long	0			! pointer to bootp block from boot rom

oldDS:	.word	0			! old DS from boot rom
oldES:	.word	0			! old ES from boot rom
oldBP:	.word	0			! old BP from boot rom
oldSI:	.word	0			! old SI from boot rom
oldDI:	.word	0			! old DI from boot rom

avlmem:	.word	0			! available amount of RAM in kB

botptr:	.long	0			! pointer to bootp record
botlen:	.word	0			! length of bootp record
botid:	.word	0			! vendor magic ID (offset into table)

partnum:.byte	0			! partition number
diskid:	.byte	0			! device id
