/*
 * first.inc  -  constants for assembler module
 *
 * Version 1.0 <Sun Jan 19 17:00:00 1997>
 *
 * Copyright (C) 1997    Gero Kuhlmann   <gero@gkminix.han.de>
 *                   and Markus Gutschke <gutschk@math.uni-muenster.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



/*
 *====================================================================
 *
 * Definitions for allocating dynamic memory, which is used to hold
 * temporary working buffers.
 */

/*
 * The MAX_MEM segment address must be within the range of addresses
 * contained in MIN_MEM number of kB.
 */
#define MIN_MEM		612		/* Minimum number of kB of base mem */
#define MAX_MEM		0x9800		/* Maximum usable segment (see SPEC.DOC) */

#define STACK_SIZE	4096		/* Size of static stack memory */

#define MAX_BOOTP_LEN	 512		/* Maximum length of bootp record */

#define MAX_DYNAMIC_LEN	MAX_BOOTP_LEN


/*
 *====================================================================
 *
 * Vendor information for the boot rom image. These values have to be
 * identical to those in mknbi.h.
 */

#define VENDOR_MAGIC	"GK-mknbi-blkdev"/* vendor ID */
#define VENDOR_SIZE	5		 /* size of vendor ID in dwords */

#define VENDOR_BOOTL	16		/* tag for boot loader segment */


/*
 *====================================================================
 *
 * Layout and magic ID of boot rom image header. These values have to
 * be identical to those used by the boot rom. See SPEC.DOC of the
 * boot rom source for further information.
 */

#define	BOOT_MAGIC	0x1B031336	/* boot image header magic cookie */
#define BOOT_SIZE	512		/* total size of boot image header */

#define BOOT_HD_SIZE	4		/* size of header in dwords */
#define BOOT_HD_MAGIC	0		/* offset for header magic number */
#define BOOT_HD_LENGTH	4		/* offset for header length */
#define BOOT_HD_FLAG1	5		/* offset for header flag 1 */
#define BOOT_HD_FLAG2	6		/* offset for header flag 2 */
#define BOOT_HD_FLAG3	7		/* offset for header flag 3 */
#define BOOT_HD_LOCN	8		/* offset for header location */
#define BOOT_HD_EXEC	12		/* offset for execute address */
#define BOOT_HD_VENDOR	16		/* offset for header vendor information */

#define BOOT_LD_SIZE	4		/* size of load record in dwords */
#define BOOT_LD_LENGTH	0		/* offset for load record length */
#define BOOT_LD_TAG1	1		/* offset for load record tag 1 */
#define BOOT_LD_TAG2	2		/* offset for load record tag 2 */
#define BOOT_LD_FLAGS	3		/* offset for load record flags */
#define BOOT_LD_ADDR	4		/* offset for absolute address */
#define BOOT_LD_ILENGTH	8		/* offset for image length */
#define BOOT_LD_MLENGTH	12		/* offset for memory length */
#define BOOT_LD_VENDOR	16		/* offset for vendor information */

#define BOOT_FLAG_B0	0x01		/* mask for load record flag B0 */
#define BOOT_FLAG_B1	0x02		/* mask for load record flag B1 */
#define BOOT_FLAG_EOF	0x04		/* mask for load record flag EOF */


/*
 *====================================================================
 *
 * Layout and magic ID of bootp record. Refer to RFC 951, RFC 1048
 * and RFC 1533 for further information. The BOOTP record can be
 * longer than the standard length as the bootrom might be able to
 * load a vendor extension file via tftp.
 */

#define BOOTP_MAGIC_RFC	0x63, 0x82, 0x53, 0x63	/* RFC 1048 vendor ID */
#define BOOTP_MAGIC_CMU	0x43, 0x4D, 0x55, 0x00	/* CMU vendor ID */
#define BOOTP_MAGIC_STA	0x53, 0x54, 0x41, 0x4E	/* Stanford vendor ID */
#define BOOTP_MAGIC_LEN	4			/* length of vendor ID */

#define BOOTP_REQUEST	1		/* bootp OP-code */
#define BOOTP_REPLY	2

#define BOOTP_OP	0		/* offset to bootp OP-code */
#define BOOTP_HTYPE	1		/* offset to hardware type */
#define BOOTP_HLEN	2		/* offset to length of hardware addr */
#define BOOTP_HOPS	3		/* offset to number of gateway hops */
#define BOOTP_XID	4		/* offset to transfer ID */
#define BOOTP_SECS	8		/* offset to seconds since boot began */
#define BOOTP_CIADDR	12		/* offset to client IP address */
#define BOOTP_YIADDR	16		/* offset to "your" IP address */
#define BOOTP_SIADDR	20		/* offset to server IP address  */
#define BOOTP_GIADDR	24		/* offset to gateway IP address */
#define BOOTP_CHADDR	28		/* offset to hardware address */
#define BOOTP_SNAME	44		/* offset to server name */
#define BOOTP_FILE	108		/* offset to name of boot file */
#define BOOTP_VEND	236		/* offset to vendor information */

#define BOOTP_SIZE	300		/* size of complete bootp record */

#define BOOTP_RFC_NOP	0		/* RFC vendor tag for NO-OP */
#define BOOTP_RFC_MSK	1		/* RFC vendor tag for netmask */
#define BOOTP_RFC_GWY	3		/* RFC vendor tag for gateway */
#define BOOTP_RFC_HNAME	12		/* RFC vendor tag for host name */
#define BOOTP_RFC_ROOT	17		/* RFC vendor tag for root directory */
#define BOOTP_RFC_VNDID	128		/* Vendor tag: magic identification */
#ifndef BOOTP_RFC_CMDAD
#define BOOTP_RFC_CMDAD	129		/* Site specific vendor tag */
#endif
#define BOOTP_RFC_VNDSEL 176		/* index to user selected image */
#define BOOTP_RFC_END	255		/* RFC vendor tag for end of record */

#define VNDID_1		0x45E4
#define VNDID_2		0x6874


/*
 *========================================================================
 *
 * Miscellaneous definitions
 *
 */

#define HD_BASE		0x80		/* base number for all harddisks */
#define FD_BASE		0x0		/* base number for all floppy disks */

#define BOOT_SIG	0xAA55		/* boot block signature */
#define BOOT_SIG_OFS	0x1FE		/* offset to boot block signature */

#define PART_OFS	0x1BE		/* offset to partition table */
#define PART_HEAD_OFS	1		/* offset to head num in part entry */
#define PART_SECT_OFS	2		/* offset to sector num in part entry*/
#define PART_TYPE_OFS	4		/* offset to partition type indicator*/

#define ENTRY_LEN	0x10		/* size of one partition table entry */
#define ENTRY_NUM	4		/* maximum num of entries in part tab*/

#define EXT_PART_TYPE	5		/* type ID for extended partition */

#define BOOTSEG		0x07C0		/* segment for loading boot sector */