/*
 * mknbi.h  -  mknbi constants and type definitions
 *
 * Version 1.0 <Sun Jan 19 00:50:00 1997>
 *
 * Copyright (C) 1997    Gero Kuhlmann   <gero@gkminix.han.de>
 *                   and Markus Gutschke <gutschk@math.uni-muenster.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _MKNBI_H_BLKDEV_
#define _MKNBI_H_BLKDEV_

#define VERSION "Version 1.0 <Sun Jan 19 17:00:00 1997>"


/*
 * Include the following two header files to allow correct handling of
 * structure fields.
 */
#include <sys/types.h>
#include <sys/param.h>

/*
 * The RAM looks like this after the boot image has been loaded by the
 * Boot-ROM:
 *
 *  0x92200-0x931FF	  4kB	primary boot loader
 *  0x93200-0x933FF	0.5kB	load header
 */

#ifndef DEF_BOOTLSEG
#  define DEF_BOOTLSEG	0x9220
#endif
#define BOOTLLSIZE	4096	/* maximum load size of primary boot loader */
#define BOOTLMSIZE	4096	/* maximum memory size of primary boot loader */

#ifndef DEF_HEADERSEG
#  define DEF_HEADERSEG	0x9320
#endif
#define HEADERLSIZE	512	/* maximum load size of boot image header */
#define HEADERMSIZE	512	/* maximum memory size of boot image header */

#define SECTSIZE	512			/* Size of one sector */

#define BOOT_SIGNATURE	0xaa55			/* boot signature */


/*
 * The boot image has the following header in it's first sector
 */
struct load_header {
  struct i_long  magic		PACKED;		/* magic number */
  __u8           hlength	PACKED;		/* length of header */
  __u8           hflags1	PACKED;		/* header flags */
  __u8           hflags2	PACKED;
  __u8           hflags3	PACKED;
  struct i_addr  locn		PACKED;		/* location of this header */
  struct i_addr  execute	PACKED;		/* execution address */
  __u8           dummy[494];			/* up to full sector */
  __u16          bootsig	PACKED;		/* boot signature */
};

#define HEADER_MAGIC	0x1B031336		/* Magic no for load header */



/*
 * In the header sector are the load records located, which actually tell
 * which part of the following image has to go where in memory
 */

union vendor_data {
  __u8           rdflags	PACKED;		/* flags for rd loading */
};

struct load_record {
  __u8              rlength	PACKED;		/* length of record */
  __u8              rtag1	PACKED;		/* record tags */
  __u8              rtag2	PACKED;
  __u8              rflags	PACKED;		/* record flags */
  struct i_long     address	PACKED;		/* abs addr for part in mem */
  struct i_long     ilength	PACKED;		/* len of part in boot image */
  struct i_long     mlength	PACKED;		/* memory needed for part */
  union vendor_data vendor_data	PACKED;		/* optional vendor data */
};

#define FLAG_B0		1
#define FLAG_B1		2
#define FLAG_EOF	4

/*
 * Number of each load record. These numbers must be in the same order
 * that the respective "initrec" calls happen!
 */
#define BOOTLNUM	0		/* primary boot loader */

#define NUM_RECORDS	1

#define VENDOR_ID	"GK-mknbi-blkdev"	/* Vendor ID */
#define VENDOR_OFF	16			/* Offset for vendor tags */



/*
 * Data area which contains the boot loader module
 */
extern char first_data[];
extern int first_data_size;


#endif
