! LZHuf (LZSS) Decompressing boot loader for ROM images
!
! this code is based on the work of Haruyasu Yoshizaki and Haruhiko Okumura
! who implemented the original compressor and decompressor in C code
!
! Copyright 1997 by M. Gutschke <gutschk@math.uni-muenster.de>
!
! Compression pays off, as soon as the uncompressed image is bigger than
! about 1.5kB. This assumes an average compressibility of about 60%.

! At entry, the processor is in 16 bit real mode and the code is being
! executed from an address it was not linked to. Code must be pic and
! 32 bit sensitive until things are fixed up.

#include "loader.inc"

	.text
_main:
#ifdef	BOOTROM
	.word	0xAA55			! BIOS extension signature
size:	.byte	0			! number of 512 byte blocks
					! = number of 256 word blocks
					! filled in by makerom program
	jmp	over			! skip over checksum
	.byte	0			! checksum 
#ifdef	NO_PCI_PNP_HEADER
	jmp	blockmove		! alternate entry point +6
					! used by floppyload
#else	/* NO_PCI_PNP_HEADER */
	br	blockmove		! alternate entry point +6
					! used by floppyload
	.ascii	'Etherboot'		! Might as well put something here
	.blkb	0x18-(*-_main)
	.word	PCI			! offset to pci data structure
	.word	PnP			! offset to PnP expansion header

PCI:
	.ascii	"PCIR"			! signature
	.word	0x8086			! vendor id (hardcode as Intel)
					! should be filled by makerom? 
	.word	0x1229			! device ID Intel 82557
					! should be filled by makerom?
	.word	0x0000			! pointer to vital product data
	.word	0x0018			! PCI data structure length 
	.byte	0			! PCI data structure revision
	.byte	0			! Class code byte 1
	.byte	0			! Class code byte 2
	.byte	0x02			! Class code byte 3 (from hexdumping
					! Intel bootrom image)
	.word	0x0000			! Image length same as offset 02h 
	.word	0x0001			! revision level of code /data
	.byte	0			! code type
	.byte	0x80			! indicator (from hexdumping 
					! Intel bootrom image)
	.word	0x0000			! reserved
      
PnP:	
	.ascii	"$PnP"			! signature
	.byte	0x01			! structure revision
	.byte	0x02			! length (in 16 byte increments)
	.word	0x0000			! offset of next header
	.byte	0			! Reserved
	.byte	0			! checksum filled by makerom
	.long	0x00000000		! Device identifier
	.word	0x0000			! pointer to manufacturer str
	.word	0x0000			! pointer to product name
	.byte	0			! device type code byte 1
	.byte	0			! device type code byte 2
	.byte	2			! device type code byte 3 (from 
					! hexdumping Intel bootrom image)
	.byte	0x14			! device indictors (from 
					! hexdumping Intel bootrom image)
	.word	0x0000			! boot connection vector
	.word	0x0000			! disconnect vector 
	.word	over			! bootstrap entry vector 
	.word	0x0000			! reserved
	.word	0x0000			! static resource information vector
#endif
over:
#ifndef	NOINT19H
	push	ax
	push	ds
	xor	ax,ax
	mov	ds,ax			! access first 64kB segment
	mov	ax,SCRATCHVEC+4		! check if already installed
	cmp	ax,#MAGIC		! check magic word
	jz	installed
	mov	ax,INT19VEC		! hook into INT19h
	mov	SCRATCHVEC,ax
	mov	ax,INT19VEC+2
	mov	SCRATCHVEC+2,ax
	mov	ax,#start19h
	mov	INT19VEC,ax
	mov	ax,cs
	mov	INT19VEC+2,ax
	mov	ax,#MAGIC		! set magic word
	mov	SCRATCHVEC+4,ax
installed:
	pop	ds
	pop	ax
	retf
start19h:				! clobber magic id, so that we will
	xor	ax,ax			!   not inadvertendly end up in an
	mov	ds,ax			!   endless loop
	mov	SCRATCHVEC+4,ax
	mov	ax,SCRATCHVEC+2		! restore original INT19h handler
	mov	INT19VEC+2,ax
	mov	ax,SCRATCHVEC
	mov	INT19VEC,ax
#endif	/* NOINT19H */
	/* fall thru */
blockmove:
#ifdef	MOVEROM	
! Relocate body of boot code to MOVEROM:0 and execute there
	cld
	mov	ax,#MOVEROM>>4
        mov     es,ax
	xor	di,di			! di = 0
	xor	si,si			! si = 0
	xor	cx,cx
	mov	ax,cs
	mov	ds,ax			! for the ref to size below
	mov	ch,size-_main
	push	cx			! save cx before decrementing
	seg	cs			! override, source = cs
        rep
        movsw

! Save ROMs CS and length in floppy boot block
	pop	cx
	push	ds
	mov	ax,#FLOPPY_SEGMENT
	mov	ds,ax
	mov	ax,cs
	mov	ROM_SEGMENT,ax
	mov	ROM_LENGTH,cx
	pop	ds

	jmpi	move,(MOVEROM>>4)
move:
#else
! Save ROMs CS and length in floppy boot block
	xor	cx,cx
	mov	ch,size-_main
	push	ds
	mov	ax,#FLOPPY_SEGMENT
	mov	ds,ax
	mov	ax,cs
	mov	ROM_SEGMENT,ax
	mov	ROM_LENGTH,cx
	pop	ds
#endif	/* MOVEROM */
#endif	/* BOOTROM */

! ds == cs
	mov	ax,cs
	mov	ds,ax
! Change stack
	mov	ax,#(RELOC-SCRATCH)>>4	! first 32kB -> scratch space
	mov	es,ax			!  new es
	mov	bx,#RELOC>>4		!  new ss
	int	#0x12			!  get conventional memory size in KB
	shl	ax, #6
	sub	ax, bx			
	shl	ax, #4			!  new sp
	cli
	mov	ss,bx
	mov	sp,ax


!----------------------------------------------------------------------------

! INIT -- initializes all data structures
! ====

init:	cld
#ifdef	BOOTROM
	mov	si,#dcodrle-_main	! uncompress run length encoded
#else	/* BOOTROM */
	mov	si,#0x100+dcodrle-_main
#endif	/* BOOTROM */
	mov	di,#dcode		!   lookup table for codes
	mov	dl,#6
	mov	dh,#0x20
	xor	bh,bh
init0:	lodsb
	mov	bl,al
init1:	mov	cl,dh
	xor	ch,ch
	mov	al,bh
	rep
	stosb
	inc	bh
	dec	bl
	jnz	init1
	shr	dh,1
	dec	dl
	jnz	init0
	mov	bl,#1			! uncompress run length encoded
	mov	bh,#6			!   lookup table for code lengths
init2:	lodsb
	mov	cl,al
	xor	ch,ch
	mov	al,bl
	rep
	stosb
	inc	bl
	dec	bh
	jnz	init2
	mov	ax,es			! we no longer have to access static
	mov	ds,ax			!    data, so set segment accordingly
	mov	cx,#NCHAR		! set all frequencies of leaf nodes
	mov	ax,#1			!   to one
	rep
	stosw
	mov	si,#freq
	mov	cx,#ROOT+1-NCHAR
init3:	lodsw				! update frequencies of non-leaf nodes
	mov	bx,ax
	lodsw
	add	ax,bx
	stosw
	loop	init3
	mov	ax,#0xFFFF
	stosw				! sentinel with infinite frequency
	mov	cx,#NCHAR
	mov	ax,#TABLESZ
init4:	stosw				! update "son" pointers for leaf nodes
	inc	ax
	loop	init4
	mov	cx,#ROOT+1-NCHAR
	xor	ax,ax
init5:	stosw				! update "son" ptrs for non-leaf nodes
	add	ax,#2
	loop	init5
	mov	cx,#ROOT+1-NCHAR
	mov	ax,#NCHAR
init6:	stosw				! update "parent" ptrs for non-leaf nd.
	stosw
	inc	ax
	loop	init6
	mov	cx,#NCHAR
	xor	ax,ax
	stosw				! root node has no parent
init7:	stosw				! update "parent" ptrs for leaf nodes
	inc	ax
	loop	init7
	xor	ax,ax
	stosb				! clear getlen
	stosw				! clear getbuf
	mov	al,#0x20		! fill text buffer with spaces
	mov	di,#spaces
	mov	cx,#BUFSZ-LOOKAHEAD
	rep
	stosb
	/* fall thru */


!----------------------------------------------------------------------------

! MAIN -- reads compressed codes and writes decompressed data
! ====

#ifdef	BOOTROM
	mov	si,#compressed-_main	! get length of compressed data stream
#else	/* BOOTROM */
	mov	si,#0x100+compressed-_main	! com offset is 0x100
#endif	/* BOOTROM */
	mov	di,#uncompressed
	seg	cs
	lodsw
	mov	cx,ax
	lodsw				! cannot do more than 64k anyways
main1:	push	cx
	call	dcdchr			! decode one code symbol
	or	ah,ah			! test if 8bit character
	jnz	main2
	stosb				! store verbatim
	pop	cx
	loop	main1			! proceed with next compressed code
	jmp	done			! until end of input is detected
main2:	push	ax
	call	dcdpos			! compute position in output buffer
	mov	ax,si
	sub	bx,di
	not	bx
	mov	si,bx			! si := di - dcdpos() - 1
	pop	cx
	sub	cx,#255-THRESHOLD	! compute length of code sequence
	mov	dx,cx
	rep
	movsb
	mov	si,ax
	pop	cx
	sub	cx,dx			! check end of input condition
	jnz	main1			! proceed with next compressed code
done:
	mov	ax,#RELOC>>4		! set ds then call etherboot
	mov	ds,ax
	callf	#0,RELOC>>4
	int	0x19

!----------------------------------------------------------------------------


! GETBIT -- gets one bit pointed to by DS:SI
! ======
! 
! changes: AX,CX,DL

getbit:	mov	cl,#8
	mov	dl,getlen		! compute number of bits required
	sub	cl,dl			!   to fill read buffer
	jae	getbit1
	mov	ax,getbuf		! there is still enough read ahead data
	jmp	getbit2
getbit1:seg	cs			! get next byte from input stream
	lodsb
	xor	ah,ah
	shl	ax,cl			! shift, so that it will fit into
	mov	cx,getbuf		!   read ahead buffer
	or	ax,cx
	add	dl,#8			! update number of bits in buffer
getbit2:mov	cx,ax
	shl	cx,1			! extract one bit from buffer
	mov	getbuf,cx
	dec	dl
	mov	getlen,dl		! and update number of bits
	shl	ax,1			! return in carry flag
	ret

	
!----------------------------------------------------------------------------

! DCDPOS -- decodes position in textbuffer as pointed to by DS:SI, result in BX
! ======
!
! changes: AX,BX,CX,DX

dcdpos:	mov	bx,#0x0800
dcdpos1:shl	bl,1			! read one byte
	call	getbit
	jnc	dcdpos2
	inc	bl
dcdpos2:dec	bh
	jnz	dcdpos1
	mov	dh,bl			! read length of code from table
	xor	bh,bh
	mov	cl,dlen[bx]
	xor	ch,ch
	mov	bl,dcode[bx]		! get top six bits from table
	shl	bx,1
	shl	bx,1
	shl	bx,1
	shl	bx,1
	shl	bx,1
	shl	bx,1
dcdpos3:push	cx			! read the rest from the input stream
	shl	dh,1
	call	getbit
	jnc	dcdpos4
	inc	dh
dcdpos4:pop	cx
	loop	dcdpos3
	and	dh,#0x3f		! combine upper and lower half of code
	or	bl,dh
	ret
	

!----------------------------------------------------------------------------

! DCDCHR -- decodes one compressed character pointed to by DS:SI
! ======
!
! changes: AX,BX,CX,DX

dcdchr:	mov	bx,#ROOT		! start at root entry
	shl	bx,1
	mov	bx,son[bx]
dcdchr1:call	getbit			! get a single bit
	jnc	dcdchr2
	inc	bx			! travel left or right
dcdchr2:shl	bx,1
	mov	bx,son[bx]
	cmp	bx,#TABLESZ		! until we come to a leaf node
	jb	dcdchr1
	mov	ax,bx
	sub	ax,#TABLESZ
	/* fall thru */

!


! UPDATE -- updates huffman tree after incrementing frequency for code in BX
! ======
! 
! changes: BX,CX,DX

update:	! we do not check whether the frequency count has overrun.
	! this will cause problems for large files, but be should be fine
	! as long as the compressed size does not exceed 32kB and we
	! cannot do more than this anyways, because we load into the
	! upper 32kB of conventional memory
	push	si
	push	ax
	shl	bx,1
	mov	bx,parent[bx]
update1:shl	bx,1
	mov	dx,freq[bx]
	inc	dx			! increment frequency count by one
	mov	freq[bx],dx
	mov	si,bx
	add	si,#freq+2
	lodsw				! check if nodes need reordering
	cmp	dx,ax
	jbe	update5
update2:lodsw
	cmp	ax,dx
	jb	update2
	mov	cx,-4[si]
	mov	freq[bx],cx		! swap frequency of entries
	mov	-4[si],dx
	mov	ax,si			! compute index of new entry
	sub	ax,#freq+4
	mov	dx,ax
	shr	ax,1
	mov	cx,son[bx]		! get son of old entry
	mov	si,cx
	add	si,si
	mov	parent[si],ax		! and update the ptr to new parent
	cmp	cx,#TABLESZ
	jae	update3			! do this for both branches
	mov	parent+2[si],ax		!   if not a leaf node
update3:mov	si,dx
	mov	dx,son[si]		! get son of new entry
	mov	son[si],cx		! update its contents
	mov	si,dx
	add	si,si
	mov	cx,bx
	shr	cx,1
	mov	parent[si],cx		! and update the ptr to new paren
	cmp	dx,#TABLESZ
	jae	update4			! do this for both branches
	mov	parent+2[si],cx		!   if not a leaf node
update4:mov	son[bx],dx		! update son of old entry
	mov	bx,ax			! continue with new entry
	shl	bx,1
update5:mov	bx,parent[bx]		! continue with parent
	or	bx,bx
	jnz	update1			! until we found the root entry
	pop	ax
	pop	si
	ret


!----------------------------------------------------------------------------

! constant data. this part of the program resides in ROM and cannot be
! changed

! run length encoded tables will be uncompressed into the bss segment
! take care with any symbols here for .com files to add 0x100 to address

dcodrle:.byte	0x01,0x03,0x08,0x0C,0x18,0x10
dlenrle:.byte	0x20,0x30,0x40,0x30,0x30,0x10

! Force 4 byte alignment
	if	((*-_main)&3) != 0
	.blkb	3-((*-_main)&3)
	.byte	0
	endif
compressed:
! compressed data will be attached here


!----------------------------------------------------------------------------

! variable data segment (bss)
! this segment will always be found at 0x90000 (i.e. at RELOC - SCRATCH)

! do not change the order or the sizes of any of the following tables
! the initialization code makes assumptions on the exact layout of the
! data structures...


! lookup table for index into buffer of recently output characters

dcode:	=	0


! lookup table for length of code sequence from buffer of recent characters

dlen:	=	dcode+256


! table with frequency counts for all codes
freq:	=	dlen+256


! pointer to child nodes
son:	=	freq+2*(TABLESZ+1)


! the first part of this table contains all the codes (0..TABLESZ-1)
! the second part contains all leaf nodes             (TABLESZ..   )
parent:	=	son+2*(TABLESZ)


! temporary storage for extracting bits from compressed data stream
getlen:	=	parent+2*(TABLESZ+NCHAR)
getbuf:	=	getlen+1


! install our stack pointer somewhere out of the way
stack:	=	SCRATCH-BUFSZ+LOOKAHEAD


! the initial buffer has to be filled with spaces

spaces:	=	stack


! uncompressed data will be written to address 0x98000

uncompressed: =	SCRATCH
