/**************************************************************************
MISC Support Routines
**************************************************************************/

#include "etherboot.h"

/**************************************************************************
TWIDDLE
**************************************************************************/
void twiddle()
{
	static unsigned long lastticks = 0;
	static int count=0;
	static char tiddles[]="-\\|/";
	unsigned long ticks;
	if ((ticks = currticks()) < lastticks)
		return;
	lastticks = ticks+1;
	putchar(tiddles[(count++)&3]);
	putchar('\b');
}

/* the b* functions are already in bcc's libc.a */
#ifdef	ETHERBOOT32
/**************************************************************************
BCOPY
**************************************************************************/
void bcopy(s,d,n)
	void *s, *d;
	int n;
{
	char *dd = d, *ss = s;
	while ((n--) > 0) {
		*(dd++) = *(ss++);
	}
}

/**************************************************************************
BZERO
**************************************************************************/
void bzero(d,n)
	void *d;
	int n;
{
	char *dd = d;
	while ((n--) > 0) {
		*(dd++) = 0;
	}
}

/**************************************************************************
BCMP
**************************************************************************/
int bcmp(d,s,n)
	void *d,*s;
	int n;
{
	char *dd = d, *ss = s;
	while ((n--) > 0) {
		if (*(dd++) != *(ss++)) return(1);
	}
	return(0);
}

/**************************************************************************
STRCASECMP (no entirely correct, but this will do for our purposes)
**************************************************************************/
int strcasecmp(a,b)
	char *a, *b;
{
	while (*a && *b && (*a & ~0x20) == (*b & ~0x20)) {a++; b++; }
	return((*a & ~0x20) - (*b & ~0x20));
}
#endif	/* ETHERBOOT32 */

/**************************************************************************
PRINTF and friends

	Formats:
		%X	- 4 byte ASCII (8 hex digits)
		%x	- 2 byte ASCII (4 hex digits)
		%b	- 1 byte ASCII (2 hex digits)
		%d	- decimal (also %i)
		%c	- ASCII char
		%s	- ASCII string
		%I	- Internet address in x.x.x.x notation
**************************************************************************/
static char hex[]="0123456789ABCDEF";
char *do_printf(buf, fmt, dp)
	char *buf, *fmt;
	int  *dp;
{
	register char *p;
	char tmp[16];
	while (*fmt) {
		if (*fmt == '%') {	/* switch() uses more space */
			fmt++;

			if (*fmt == 'X') {
				long *lp = (long *)dp;
				register long h = *lp++;
				dp = (int *)lp;
				*(buf++) = hex[(h>>28)& 0x0F];
				*(buf++) = hex[(h>>24)& 0x0F];
				*(buf++) = hex[(h>>20)& 0x0F];
				*(buf++) = hex[(h>>16)& 0x0F];
				*(buf++) = hex[(h>>12)& 0x0F];
				*(buf++) = hex[(h>>8)& 0x0F];
				*(buf++) = hex[(h>>4)& 0x0F];
				*(buf++) = hex[h& 0x0F];
			}
			if (*fmt == 'x') {
				register int h = *(dp++);
				*(buf++) = hex[(h>>12)& 0x0F];
				*(buf++) = hex[(h>>8)& 0x0F];
				*(buf++) = hex[(h>>4)& 0x0F];
				*(buf++) = hex[h& 0x0F];
			}
			if (*fmt == 'b') {
				register int h = *(dp++);
				*(buf++) = hex[(h>>4)& 0x0F];
				*(buf++) = hex[h& 0x0F];
			}
			if ((*fmt == 'd') || (*fmt == 'i')) {
				register int dec = *(dp++);
				p = tmp;
				if (dec < 0) {
					*(buf++) = '-';
					dec = -dec;
				}
				do {
					*(p++) = '0' + (dec%10);
					dec = dec/10;
				} while(dec);
				while ((--p) >= tmp) *(buf++) = *p;
			}
			if (*fmt == 'I') {
				long *lp = (long *)dp;
				register long h = *lp++;
				dp = (int *)lp;
				buf = sprintf(buf,"%d.%d.%d.%d",
					(int)(h>>24) & 0x00FF,
					(int)(h>>16) & 0x00FF,
					(int)(h>>8) & 0x00FF,
					(int)h & 0x00FF);
			}
			if (*fmt == 'c')
				*(buf++) = *(dp++);
			if (*fmt == 's') {
				p = (char *)*dp++;
				while (*p) *(buf++) = *p++;
			}
		} else *(buf++) = *fmt;
		fmt++;
	}
	*buf = 0;
	return(buf);
}

char *sprintf(buf, fmt, data)
	char *fmt, *buf;
	int data;
{
	return(do_printf(buf,fmt, &data));
}

void printf(fmt,data)
	char *fmt;
	int data;
{
	char buf[120],*p;
	p = buf;
	do_printf(buf,fmt,&data);
	while (*p) putchar(*p++);
}

#ifdef	IMAGE_MENU
/**************************************************************************
SETIP - Convert an ascii x.x.x.x to binary form
**************************************************************************/
int setip(p, i)
	char *p;
	unsigned long *i;
{
	unsigned long ip = 0;
	int val;
	if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
	if (*p != '.') return(0);
	p++;
	ip = val;
	if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
	if (*p != '.') return(0);
	p++;
	ip = (ip << 8) | val;
	if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
	if (*p != '.') return(0);
	p++;
	ip = (ip << 8) | val;
	if (((val = getdec(&p)) < 0) || (val > 255)) return(0);
	*i = (ip << 8) | val;
	return(1);
}

#endif	/* IMAGE_MENU */

int getdec(ptr)
	char **ptr;
{
	char *p = *ptr;
	int ret=0;
	if ((*p < '0') || (*p > '9')) return(-1);
	while ((*p >= '0') && (*p <= '9')) {
		ret = ret*10 + (*p - '0');
		p++;
	}
	*ptr = p;
	return(ret);
}

#define K_RDWR 		0x60		/* keyboard data & cmds (read/write) */
#define K_STATUS 	0x64		/* keyboard status */
#define K_CMD	 	0x64		/* keybd ctlr command (write-only) */

#define K_OBUF_FUL 	0x01		/* output buffer full */
#define K_IBUF_FUL 	0x02		/* input buffer full */

#define KC_CMD_WIN	0xd0		/* read  output port */
#define KC_CMD_WOUT	0xd1		/* write output port */
#define KB_A20		0xdf		/* enable A20,
					   enable output buffer full interrupt
					   enable data line
					   disable clock line */
#ifndef	IBM_L40

static void empty_8042(void)
{
	extern void slowdownio();
	unsigned long time;
	char st;
  
  	slowdownio();
	time = currticks() + 18;	/* max wait of 1 second */
  	while ((((st = inb(K_CMD)) & K_OBUF_FUL) ||
	       (st & K_IBUF_FUL)) &&
	       currticks() < time)
		inb(K_RDWR);
}
#endif	IBM_L40

/*
 * Gate A20 for high memory
 */
void gateA20()
{
#ifdef	IBM_L40
	OUTB(0x92, 0x2);
#else	IBM_L40
	empty_8042();
	OUTB(K_CMD, KC_CMD_WOUT);
	empty_8042();
	OUTB(K_RDWR, KB_A20);
	empty_8042();
#endif	IBM_L40
}

/*
 * Local variables:
 *  c-basic-offset: 8
 * End:
 */
